#include <stdlib.h>
#include "list.h"

/*** define global function ***/

//////////////////////////////////////////
// list_new()
//////////////////////////////////////////
LIST *list_new(int (*compare)(const void *, const void *))
{
	LIST *list = NULL;
	LIST_NODE *head = NULL;
	LIST_NODE *tail = NULL;

	if((list = (LIST *)malloc(sizeof(LIST))) == NULL) {
		return NULL;
	}
	if((head = (LIST_NODE *)malloc(sizeof(LIST_NODE))) == NULL) {
		return NULL;
	}
	if((tail = (LIST_NODE *)malloc(sizeof(LIST_NODE))) == NULL) {
		return NULL;
	}
	
	head->r_link = tail; head->l_link = NULL;
	tail->r_link = NULL; tail->l_link = head;
	
	// set default
	list->node = 0;
	list->compare = compare;
	list->head = head;
	list->tail = tail;

	return list;
}

//////////////////////////////////////////
// list_delete()
//////////////////////////////////////////
void list_delete(LIST *list)
{
	LIST_NODE *tmp = list->head->r_link;
	
	for(;tmp != list->tail; tmp = tmp->r_link) {
		free(tmp->record);
		free(tmp);
	}

	free(list->head);
	free(list->tail);
	free(list);
}

/////////////////////////////////////////
// list_insert_node()
/////////////////////////////////////////
LIST *list_insert_node(LIST *list, const void *record)
{
	LIST_NODE *tmp = list->head->r_link;
	LIST_NODE *node = node;

	if((node = (LIST_NODE *)malloc(sizeof(LIST_NODE))) == NULL) {
		return NULL;
	}
	
	for(;tmp != list->tail; tmp = tmp->r_link) {
		if(list->compare(tmp->record, record) > 0) break;
	}

	node->record = (void *)record;
	node->r_link = tmp;
	node->l_link = tmp->l_link;
	tmp->l_link = node;
	node->l_link->r_link = node;

	(list->node)++;
	return list;
}

/////////////////////////////////////////
// list_del_node()
/////////////////////////////////////////
LIST *list_del_node(LIST *list, void *record)
{
	LIST_NODE *tmp = list->head->r_link;

	for(;tmp != list->tail; tmp = tmp->r_link) {
		if(!(list->compare(tmp->record, record))) {
			tmp->l_link->r_link = tmp->r_link;
			tmp->r_link->l_link = tmp->l_link;

			free(tmp->record);
			free(tmp);

			// success
			(list->node)--;
			return list;
		}
	}

	// fail
	return NULL;
}

/////////////////////////////////////////
// list_search()
/////////////////////////////////////////
LIST_NODE *list_search(LIST *list, const void *record)
{
	LIST_NODE *tmp = list->head->r_link;

	for(;tmp != list->tail; tmp = tmp->r_link) {
		if(!(list->compare(tmp->record, record))) {
			return tmp;
		}
	}

	return NULL;
}

//////////////////////////////////////////
// list_get_node()
//////////////////////////////////////////
LIST_NODE *list_get_node(LIST *list, int n)
{
	LIST_NODE *tmp = list->head;
	
	if(list->node < n+1) return NULL;
	
	for(;n >= 0; n--) {
		tmp = tmp->r_link;
	}
	return tmp;
}

//////////////////////////////////////////
// list_size()
//////////////////////////////////////////
int list_size(LIST *list)
{
	return list->node;
}
